
const domReady = require('domready');
const dateFormat = require('dateformat');
const util = require('util');

const sensorsToCloud = require('./sensorsToCloud');
const sensorsFromSPCU = require('./sensorsFromSPCU');
const FileInfo = require('./FileInfo');
const jsonConfig = require('./jsonConfig');

const cfg_filename = './odc.json';
const fs = require("fs");


var cfg = JSON.parse(fs.readFileSync(cfg_filename));
const default_page = 'do_config';
const mode = {
    download: 0,
    upload: 1
};

function dpyPage(page, ev) {
    if (dpyPage.job) {
        dpyPage.job();
        dpyPage.job = null;
    }
    dpyPage.job = page.onEndPage.bind(page);
    dpyPage.page = page;
}

function dpyMsg(_msg) {
    var msg = util.format.apply(this, _msg);
    var date_str = dateFormat(new Date(), 'dd/mm/yyyy HH:MM:ss');
    var logUI = this.page_node.querySelector('.log');
    logUI.innerHTML = "<div><div class='log_date'>" + date_str + "</div><div>" + msg + "</div></div>" + logUI.innerHTML;
    //logUI.scrollTop = logUI.scrollHeight;
}

function ui_page(ev_name, ctx) {
    var _self = this;
    this.ev_name = ev_name;
    this.node_menu = document.querySelector('.menu .right .' + this.ev_name + '.sub');
    this.page_node = document.querySelector('#page_' + this.ev_name);
    this.ctx = ctx;
    this.ctx.parent = this;

    this.onStartPage = function (ev) {
        dpyPage(this, ev);
        if (this.ctx.onStartPage) this.ctx.onStartPage(this, cfg, ev);
    };
    this.onEndPage = function () {

        if (this.ctx.onEndPage) this.ctx.onEndPage(this, cfg);
    };

    this.ctx.menu.left.forEach(function (menu_class) {
        document.querySelector('.menu .left a#do_page_' + menu_class).addEventListener('click', _self.onStartPage.bind(_self));
    });

    this.ctx.menu.right.forEach(function (item_menu) {
        _self.node_menu.querySelector('a.' + item_menu.butClass).addEventListener(item_menu.ev ? item_menu.ev : 'click', item_menu.action.bind(_self));
    });
    if (this.ctx.onInitPage) this.ctx.onInitPage(this, cfg);
}

function tabSensor() {
    var sstats_div = document.querySelector('#page_' + this.ev_name + ' .sensor_stats');
    var nb_items_tot_recv = 0;
    var nb_items_tot_sent = 0;
    sstats_div.innerHTML = "<div class='title'><div>Sensor</div><div>Serial</div><div>Received</div><div>Sent</div></div>";
    FileInfo.getSensorStats(function (item) {

        var html = util.format("<div><div>%s</div><div>%s</div><div>%s</div><div>%s</div></div>", "DAU " + item.dau_id + " " + item.stype + " " + item.sid, item.serial, item.nb_items_recv,
            item.nb_items_sent);
        nb_items_tot_recv += item.nb_items_recv;
        nb_items_tot_sent += item.nb_items_sent;
        sstats_div.innerHTML += html;
    });
    sstats_div.innerHTML += "<div class='tot'><div></div><div></div><div>" + nb_items_tot_recv + "</div><div>" + nb_items_tot_sent + "</div></div>";
}

function loadUI() {
    return action = [
        /* 
        new ui_page('config', {
            onStartPage: function (page, cfg, ev) {
                page.page_node.classList.remove('hide');
                page.node_menu.classList.remove('hide');
                jsonConfig.initEditor(document.getElementById('cfg_editor'), cfg, console);
                document.getElementById('do_page_config').classList.add('select');
            },
            onEndPage: function (page, cfg) {
                page.page_node.classList.add('hide');
                page.node_menu.classList.add('hide');
                document.getElementById('do_page_config').classList.remove('select');
                console.log('Config end.');
            },
            menu: {
                left: ['config'],
                right: [{
                    butClass: 'do_reload',
                    action: function () {
                        jsonConfig.initEditor(document.getElementById('cfg_editor'), cfg, console);
                    }
                }, {
                    butClass: 'do_save',
                    action: function () {
                        console.log(this.name);
                        if (!document.querySelector('button.jsoneditor-schema-error')) {
                            cfg = jsonConfig.saveEditorConfig(cfg_filename);
                            alert('Configuration ends with success');
                        } else {
                            alert("Error: Can't save your configuration.\n Please check values");
                        }
                    }
                }],
            },
        }),
        */
        new ui_page('viewer', {
            onStartPage: function (page, cfg, ev) {
                page.page_node.classList.remove('hide');
                page.node_menu.classList.remove('hide');
                page.ctx.started = false;
                page.node_menu.querySelector('.start').classList.remove('hide');
                page.node_menu.querySelector('.stop').classList.add('hide');
                tabSensor.bind(page)();
                page.ctx.evUpdateSensor = tabSensor.bind(page);
                document.addEventListener('update_sensors', page.ctx.evUpdateSensor);
                // when clicked comes from clickev.target.id is the icon, else it is the <a> balise
                var id = (ev.target.id == "") ? ev.target.parentElement.id : ev.target.id;
                if (ev)
                    page.ctx.mode = (id == 'do_page_download') ? mode.download : mode.upload;
                else
                    page.ctx.mode = mode.download;
                if (page.ctx.mode == mode.upload) {
                    document.getElementById('do_page_download').classList.remove('select');
                    document.getElementById('do_page_upload').classList.add('select');
                    document.querySelector('.dl_menu').classList.add('hide');
                    document.querySelector('.ul_menu').classList.remove('hide');
                    page.ctx.evAllSent = page.ctx.onAllDataSent.bind(page);
                    document.addEventListener('sensors_all_sent', page.ctx.evAllSent);
                } else {
                    document.querySelector('.dl_menu').classList.remove('hide');
                    document.querySelector('.ul_menu').classList.add('hide');
                    document.getElementById('do_page_upload').classList.remove('select');
                    document.getElementById('do_page_download').classList.add('select');

                }
            },
            onEndPage: function (page) {
                page.page_node.classList.add('hide');
                page.node_menu.classList.add('hide');
                page.ctx.started = !page.ctx.started;
                if (page.ctx.mode == mode.upload)
                    document.getElementById('do_page_upload').classList.remove('select');
                else
                    document.getElementById('do_page_download').classList.remove('select');
                page.ctx.stopServer(page);
                if (page.ctx.evUpdateSensor) {
                    document.removeEventListener('update_sensors', page.ctx.evUpdateSensor);
                    page.ctx.evUpdateSensor = null;
                }
                if (page.ctx.evAllSent) {
                    document.removeEventListener('sensors_all_sent', page.ctx.evAllSent);
                    page.ctx.evAllSent = null;
                }
            },
            stopServer: function (page) {
                page.node_menu.querySelector('.start').classList.remove('hide');
                page.node_menu.querySelector('.stop').classList.add('hide');

                if (page.ctx.mode == mode.upload) {
                    sensorsToCloud.stopSent();
                } else {
                    sensorsFromSPCU.stopServer();
                }

            },
            onAllDataSent: function () {
                this.ctx.stopServer(this);
                FileInfo.removeAll();
                alert('All datas had been sent to server');
            },
            evAllSent: null,
            evUpdateSensor: null,
            started: false,
            mode: mode.download,
            menu: {
                left: ['download', 'upload'],
                right: [{
                    butClass: 'do_clear',
                    action: function () {
                        this.page_node.querySelector('.log').innerHTML = "";
                    }
                }, {
                    butClass: 'do_toggle_start',
                    action: function () {
                        this.ctx.started = !this.ctx.started;
                        if (this.ctx.started) {
                            if (cfg.path.length == 0 || cfg.spcu.port.length == 0) {
                                alert('Error Path and Connection Port must be set.');
                                if (cfg.path.length == 0) document.getElementById('path').focus();
                                else document.getElementById('port').focus();
                                return;
                            }
                            this.node_menu.querySelector('.start').classList.add('hide');
                            this.node_menu.querySelector('.stop').classList.remove('hide');

                            if (this.ctx.mode == mode.download)
                                sensorsFromSPCU.startServer(cfg, dpyMsg.bind(this), document);
                            else {
                                if (FileInfo.checkAllSent()) {
                                    this.ctx.onAllDataSent.bind(this)();
                                } else
                                    sensorsToCloud.sendToSensorStream(cfg, dpyMsg.bind(this), document, window.console);
                            }
                        } else {
                            this.ctx.stopServer(this);
                        }
                    }
                }],
            },
        })
    ];
}

function initCfg() {
    FileInfo.load(cfg.path, cfg_filename, document);
    jsonConfig.initCfg(cfg);
}

function setInputErr(iinput, err) {
    iinput.classList.add('error');
    alert("Error:" + err.value);
    iinput.focus();
}

function initUICfgValue() {
    document.getElementById('path').value = cfg.path;
    document.getElementById('port').value = cfg.spcu.port;
    document.getElementById('nb_per_sframe').value = cfg.cloud.nb_data_per_frame;
    document.getElementById('period').value = cfg.cloud.frequency;
}
domReady(function () {
    console.log("DOM is ready !!");
    initCfg();
    loadUI();
    [].forEach.call(document.querySelectorAll('input'), function (iinput) {
        iinput.addEventListener('change', function (ev) {
            var err = {};
            var prop_name;
            console.log(ev.target.id);
            switch (ev.target.id) {
                case 'vpath':
                    if (!jsonConfig.updateDlCfg(cfg_filename, 'path', this.value, err)) setInputErr(document.getElementById(path), err);
                    break;
                case 'port':
                    if (!jsonConfig.updateDlCfg(cfg_filename, ev.target.id, this.value, err)) setInputErr(ev.target, err);
                    break;
                case 'nb_per_sframe':
                case 'period':
                    if (!jsonConfig.updateUlCfg(cfg_filename, ev.target.id, this.value, err)) setInputErr(ev.target, err);
                    break;
            }
            if (!err.value) {
                ev.target.classList.remove('error');
            }
        });
    });
    initUICfgValue();
    document.getElementById('vpath').addEventListener('change', function (ev) {
        document.getElementById('path').value = this.value;
    });

    document.getElementById('path_select').addEventListener('click', function (ev) {
        document.getElementById('vpath').click();
    });


    document.querySelector('#do_page_download').click();
});
